@@ -57,6 +57,20 @@ struct keyword_enum_value kw_keyexchange_values[]={
 struct keyword_enum_values kw_keyexchange_list=
     { kw_keyexchange_values, sizeof(kw_keyexchange_values)/sizeof(struct keyword_enum_value)};
 
+/*
+ * Values for Four-State options, such as ikev2
+ */
+struct keyword_enum_value kw_fourvalued_values[]={
+    { "never",     fo_never  },
+    { "permit",    fo_permit },
+    { "propose",   fo_propose},
+    { "insist",    fo_insist },
+    { "yes",       fo_propose},
+    { "always",    fo_insist },
+    { "no",        fo_never  }
+};
+struct keyword_enum_values kw_fourvalued_list=
+{ kw_fourvalued_values, sizeof(kw_fourvalued_values)/sizeof(struct keyword_enum_value)};
 
 /*
  * Values for authby={rsasig, secret}
@@ -97,6 +111,17 @@ struct keyword_enum_value kw_auto_values[]={
 struct keyword_enum_values kw_auto_list=
     { kw_auto_values, sizeof(kw_auto_values)/sizeof(struct keyword_enum_value)};
 
+/*
+ * Values for connaddrfamily={ipv4,ipv6}
+ */
+struct keyword_enum_value kw_connaddrfamily_values[]={
+    { "ipv4",   AF_INET },
+    { "ipv6",   AF_INET6 },
+};
+
+struct keyword_enum_values kw_connaddrfamily_list=
+    { kw_connaddrfamily_values, sizeof(kw_connaddrfamily_values)/sizeof(struct keyword_enum_value)};
+
 /*
  * Values for type={tunnel,transport,udpencap}
  */
@@ -292,9 +317,11 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     {"auto",           kv_conn|kv_duplicateok, kt_enum,   KBF_AUTO,        &kw_auto_list},
     {"also",           kv_conn,         kt_appendstring, KSF_ALSO,NOT_ENUM},
     {"alsoflip",       kv_conn,         kt_string, KSF_ALSOFLIP,NOT_ENUM},
+    {"connaddrfamily", kv_conn,         kt_enum,   KBF_CONNADDRFAMILY,     &kw_connaddrfamily_list},
     {"type",           kv_conn,         kt_enum,   KBF_TYPE,        &kw_type_list},
     {"authby",         kv_conn|kv_auto, kt_enum,   KBF_AUTHBY,     &kw_authby_list},
     {"keyexchange",    kv_conn|kv_auto, kt_enum,   KBF_KEYEXCHANGE, &kw_keyexchange_list},
+    {"ikev2",          kv_conn|kv_auto|kv_processed,kt_enum,KBF_IKEv2,&kw_fourvalued_list},
     {"pfs",            kv_conn|kv_auto, kt_bool,   KBF_PFS,          NOT_ENUM},
     {"keylife",        kv_conn|kv_auto|kv_alias, kt_time,   KBF_SALIFETIME,NOT_ENUM},
     {"lifetime",       kv_conn|kv_auto|kv_alias, kt_time,   KBF_SALIFETIME,NOT_ENUM},
