@@ -39,26 +39,50 @@
 struct oakley_group_desc;
 
 /* sadb/ESP aa attrib converters */
-int
-alg_info_esp_aa2sadb(int auth)
+enum ipsec_authentication_algo
+alg_info_esp_aa2sadb(enum ikev1_auth_attribute auth)
 {
-	int sadb_aalg=0;
 	switch(auth) {
 		case AUTH_ALGORITHM_HMAC_MD5:
+		    return AH_MD5;
 		case AUTH_ALGORITHM_HMAC_SHA1:
-			sadb_aalg=auth+1;
-			break;
+		    return AH_SHA;
+
 		case AUTH_ALGORITHM_HMAC_SHA2_256:
+		    return AH_SHA2_256;
 		case AUTH_ALGORITHM_HMAC_SHA2_384:
+		    return AH_SHA2_384;
 		case AUTH_ALGORITHM_HMAC_SHA2_512:
+		    return AH_SHA2_512;
 		case AUTH_ALGORITHM_HMAC_RIPEMD:
-			sadb_aalg=auth;
-			break;
+		    return AH_RIPEMD;
+
 		default:
-			/* loose ... */
-			sadb_aalg=auth;
+		    bad_case(auth);
 	}
-	return sadb_aalg;
+	return 0;
+}
+
+/*
+ * should change all algorithms to use IKEv2 numbers, and translate
+ * at edges only
+ */
+enum ikev1_auth_attribute
+alg_info_esp_v2tov1aa(enum ikev2_trans_type_integ ti)
+{
+    switch(ti) {
+    case IKEv2_AUTH_HMAC_MD5_96:
+	return AUTH_ALGORITHM_HMAC_MD5;
+    case IKEv2_AUTH_HMAC_SHA1_96:
+	return AUTH_ALGORITHM_HMAC_SHA1;
+
+    case IKEv2_AUTH_DES_MAC:
+    case IKEv2_AUTH_KPDK_MD5:
+    case IKEv2_AUTH_AES_XCBC_96:
+    case IKEv2_AUTH_INVALID:
+	bad_case(ti);
+    }
+    return 0;
 }
 
 int /* __attribute__ ((unused)) */
@@ -871,37 +895,22 @@ alg_info_snprint(char *buf, int buflen
 			    , (int)esp_info->esp_ealg_keylen
 			    , enum_name(&auth_alg_names, esp_info->esp_aalg_id)+sizeof("AUTH_ALGORITHM_HMAC")
 			    , esp_info->esp_aalg_id);
-		if(np < buflen) {
-			ptr+=np;
-			buflen-=np;
-		} else {
-			ptr+=buflen;
-			buflen=0;
-               }
+		ptr+=np;
+		buflen-=np;
 		if ( cnt > 0) {
 			np=snprintf(ptr, buflen, ", ");
-			if(np < buflen) {
-				ptr+=np;
-				buflen-=np;
-			} else {
-				ptr+=buflen;
-				buflen=0;
-			}
+			ptr+=np;
+			buflen-=np;
 		}
-		if(buflen <= 0) goto out;
+		if(buflen<0) goto out;
 	    }
 	    if (alg_info_esp->esp_pfsgroup) {
 		np=snprintf(ptr, buflen, "; pfsgroup=%s(%d)"
 			, enum_name(&oakley_group_names, alg_info_esp->esp_pfsgroup)+ sizeof("OAKLEY_GROUP")
 		        , alg_info_esp->esp_pfsgroup);
-		if(np < buflen) {
-			ptr+=np;
-			buflen-=np;
-		} else {
-			ptr+=buflen;
-			buflen=0;
-		}
-		if(buflen <= 0) goto out;
+		ptr+=np;
+		buflen-=np;
+		if(buflen<0) goto out;
 	    }
 	    break;
 	}
@@ -916,26 +925,16 @@ alg_info_snprint(char *buf, int buflen
 			    , (int)esp_info->esp_ealg_keylen
 			    , enum_name(&auth_alg_names, esp_info->esp_aalg_id)+sizeof("AUTH_ALGORITHM_HMAC")
 			    , esp_info->esp_aalg_id);
-		if(np < buflen) {
-			ptr+=np;
-			buflen-=np;
-		} else {
-			ptr+=buflen;
-			buflen=0;
-		}
-		if(buflen <= 0) goto out;
+		ptr+=np;
+		buflen-=np;
+		if(buflen<0) goto out;
 	    }
 	    if (alg_info_esp->esp_pfsgroup) {
 		np=snprintf(ptr, buflen, "; pfsgroup=%d; "
 			    , alg_info_esp->esp_pfsgroup);
-		if(np < buflen) {
-			ptr+=np;
-			buflen-=np;
-		} else {
-			ptr+=buflen;
-			buflen=0;
-		}
-		if(buflen <= 0) goto out;
+		ptr+=np;
+		buflen-=np;
+		if(buflen<0) goto out;
 	    }
 	    break;
         }
@@ -951,24 +950,14 @@ alg_info_snprint(char *buf, int buflen
 			    , ike_info->ike_halg
 			    , enum_name(&oakley_group_names, ike_info->ike_modp)+ sizeof("OAKLEY_GROUP")
 			    , ike_info->ike_modp);
-		if(np < buflen) {
-			ptr+=np;
-			buflen-=np;
-		} else {
-			ptr+=buflen;
-			buflen=0;
-               }
+		ptr+=np;
+		buflen-=np;
                 if ( cnt > 0) {
                         np=snprintf(ptr, buflen, ", ");
-			if(np < buflen) {
-				ptr+=np;
-				buflen-=np;
-			} else {
-				ptr+=buflen;
-				buflen=0;
-                        }
+                        ptr+=np;
+                        buflen-=np;
                 }
-		if(buflen <= 0) goto out;
+		if(buflen<0) goto out;
 	    }
 	    break;
 	}
@@ -977,27 +966,15 @@ alg_info_snprint(char *buf, int buflen
     default:
 	np=snprintf(buf, buflen, "INVALID protoid=%d\n",
 		    alg_info->alg_info_protoid);
-	if(np < buflen) {
-		ptr+=np;
-		buflen-=np;
-	} else {
-		ptr+=buflen;
-		buflen=0;
-        }
+	ptr+=np;
+	buflen-=np;
 	goto out;
     }
-    if(buflen > 0){
-	np=snprintf(ptr, buflen, "; flags=%s",
+    np=snprintf(ptr, buflen, "; flags=%s",
 		alg_info->alg_info_flags&ALG_INFO_F_STRICT?
 		"strict":"-strict");
-    	if(np < buflen) {
-		ptr+=np;
-		buflen-=np;
-    	} else {
-		ptr+=buflen;
-		buflen=0;
-    	}
-    }
+    ptr+=np;
+    buflen-=np;
 
  out:
     passert(buflen >= 0);
