@@ -33,14 +33,22 @@
 
 /* version */
 
-static const char *const version_name[] = {
-	"ISAKMP Version 1.0",
+static const char *const version_name_1[] = {
+	"ISAKMP Version 1.0 (rfc2407)",
+};
+static const char *const version_name_2[] = {
+	"IKEv2 version 2.0 (rfc4306)",
 };
 
-enum_names version_names =
+enum_names version_names_1 =
     { ISAKMP_MAJOR_VERSION<<ISA_MAJ_SHIFT | ISAKMP_MINOR_VERSION,
 	ISAKMP_MAJOR_VERSION<<ISA_MAJ_SHIFT | ISAKMP_MINOR_VERSION,
-	version_name, NULL };
+	version_name_1, NULL };
+
+enum_names version_names =
+    { IKEv2_MAJOR_VERSION<<ISA_MAJ_SHIFT | IKEv2_MINOR_VERSION,
+	IKEv2_MAJOR_VERSION<<ISA_MAJ_SHIFT | IKEv2_MINOR_VERSION,
+	version_name_2, &version_names_1 };
 
 /* Domain of Interpretation */
 
@@ -133,12 +141,39 @@ const char *const payload_name[] = {
 	NULL
     };
 
-const char *const payload_name_nat_d[] = { "ISAKMP_NEXT_NAT-D",
-	"ISAKMP_NEXT_NAT-OA", NULL };
+const char *const payload_names_ikev2[] = {
+    "ISAKMP_NEXT_v2SA",            /* 33 */
+    "ISAKMP_NEXT_v2KE",
+    "ISAKMP_NEXT_v2IDi",
+    "ISAKMP_NEXT_v2IDr",
+    "ISAKMP_NEXT_v2CERT",  
+    "ISAKMP_NEXT_v2CERTREQ",
+    "ISAKMP_NEXT_v2AUTH",
+    "ISAKMP_NEXT_v2Ni",
+    "ISAKMP_NEXT_v2N",
+    "ISAKMP_NEXT_v2D",
+    "ISAKMP_NEXT_v2V",
+    "ISAKMP_NEXT_v2TSi",
+    "ISAKMP_NEXT_v2TSr",
+    "ISAKMP_NEXT_v2E",
+    "ISAKMP_NEXT_v2CP",
+    "ISAKMP_NEXT_v2EAP",
+    NULL
+};
+
+const char *const payload_name_nat_d[] = {
+    "ISAKMP_NEXT_NAT-D",
+    "ISAKMP_NEXT_NAT-OA",
+    NULL
+};
+
 static enum_names payload_names_nat_d =
-	{ ISAKMP_NEXT_NATD_DRAFTS, ISAKMP_NEXT_NATOA_DRAFTS, payload_name_nat_d, NULL };
+{ ISAKMP_NEXT_NATD_DRAFTS, ISAKMP_NEXT_NATOA_DRAFTS, payload_name_nat_d, NULL };
+static enum_names payload_names_ikev2_d =
+{ ISAKMP_NEXT_v2SA, ISAKMP_NEXT_v2EAP, payload_names_ikev2, &payload_names_nat_d };
+
 enum_names payload_names =
-    { ISAKMP_NEXT_NONE, ISAKMP_NEXT_NATOA_RFC, payload_name, &payload_names_nat_d };
+{ ISAKMP_NEXT_NONE, ISAKMP_NEXT_NATOA_RFC, payload_name, &payload_names_ikev2_d };
 
 
 /* Exchange types (note: two discontinuous ranges) */
@@ -156,19 +191,26 @@ static const char *const exchange_name[] = {
 static const char *const exchange_name2[] = {
 	"ISAKMP_XCHG_QUICK",
 	"ISAKMP_XCHG_NGRP",
-	"ISAKMP_XCHG_ACK_INFO",
+	"ISAKMP_v2_SA_INIT",
+	"ISAKMP_v2_AUTH",
+	"ISAKMP_v2_CHILD_SA",
+	"ISAKMP_v2_INFORMATIONAL",
     };
 
 static enum_names exchange_desc2 =
-    { ISAKMP_XCHG_QUICK, ISAKMP_XCHG_ACK_INFO, exchange_name2, NULL };
+    { ISAKMP_XCHG_QUICK, ISAKMP_v2_INFORMATIONAL, exchange_name2, NULL };
 
 enum_names exchange_names =
     { ISAKMP_XCHG_NONE, ISAKMP_XCHG_MODE_CFG, exchange_name, &exchange_desc2 };
 /* Flag BITS */
 const char *const flag_bit_names[] = {
-	"ISAKMP_FLAG_ENCRYPTION",
-	"ISAKMP_FLAG_COMMIT",
-	NULL
+    "ISAKMP_FLAG_ENCRYPTION",         /* bit 0 */
+    "ISAKMP_FLAG_COMMIT",             /* bit 1 */
+    "bit 2",                          /* bit 2 */
+    "ISAKMP_FLAG_INIT",               /* bit 3 */
+    "ISAKMP_FLAG_VERSION",            /* bit 4 */
+    "ISAKMP_FLAG_RESPONSE",           /* bit 5 */
+    NULL
     };
 
 /* Situation BITS definition for IPsec DOI */
@@ -298,6 +340,33 @@ static const char *const cert_type_name[] = {
 enum_names cert_type_names =
     { CERT_NONE, CERT_X509_ATTRIBUTE, cert_type_name, NULL };
 
+/* Certificate type values RFC 4306 3.6*/
+
+static const char *const ikev2_cert_type_name[] = {
+        "CERT_RESERVED",
+        "CERT_PKCS7_WRAPPED_X509",
+        "CERT_PGP",
+        "CERT_DNS_SIGNED_KEY",
+	"CERT_X509_SIGNATURE",
+	"CERT_UNUSED", /* 5 is missing did IETF drop it ? it was in IKEv1 RFC2408 */
+	"CERT_KERBEROS_TOKENS",
+	"CERT_CRL",
+	"CERT_ARL",
+	"CERT_SPKI",
+	"CERT_X509_ATTRIBUTE",
+	"CERT_RAW_RSA",
+	"CERT_X509_CERT_URL",
+	"CERT_X509_BUNDLE_URL", /* 13 */
+
+	/* AA How do I add thse ?
+	   RESERVED to IANA                  14 - 200
+           PRIVATE USE                      201 - 255
+	*/
+};
+
+enum_names ikev2_cert_type_names =
+    { CERT_NONE, CERT_X509_ATTRIBUTE, ikev2_cert_type_name, NULL };
+
 /*
  * certificate request payload policy
  */
@@ -379,6 +448,7 @@ enum_names *oakley_attr_val_descs[] = {
 	NULL,			/* OAKLEY_FIELD_SIZE */
 	NULL,			/* OAKLEY_GROUP_ORDER */
     };
+const unsigned int oakley_attr_val_descs_size = elemsof(oakley_attr_val_descs);
 
 /* IPsec DOI attributes (RFC 2407 "IPsec DOI" section 4.5) */
 
@@ -427,6 +497,7 @@ enum_names *ipsec_attr_val_descs[] = {
 	NULL,			/* COMPRESS_DICT_SIZE */
 	NULL,			/* COMPRESS_PRIVATE_ALG */
     };
+const unsigned int ipsec_attr_val_descs_size=elemsof(ipsec_attr_val_descs);
 
 /* SA Lifetime Type attribute */
 
@@ -655,6 +726,16 @@ enum_names oakley_auth_names =
     { XAUTHInitPreShared, XAUTHRespRSARevisedEncryption
 	, oakley_auth_name3, &oakley_auth_names2 };
 
+/* ikev2 auth methods */
+static const char *const ikev2_auth_strings[]={
+	"v2_AUTH_RSA",
+	"v2_AUTH_SHARED",
+	"v2_AUTH_DSA",
+};
+enum_names ikev2_auth_names =
+{ v2_AUTH_RSA, v2_AUTH_DSA, ikev2_auth_strings, NULL};
+
+
 /* Oakley Group Description attribute */
 
 static const char *const oakley_group_name[] = {
@@ -774,6 +855,89 @@ const char *const attr_msg_type_name[] = {
 enum_names attr_msg_type_names =
     { 0 , ISAKMP_CFG_ACK, attr_msg_type_name , NULL };
 
+/*
+ * IKEv2 CRITICAL BYTE "enum"
+ */
+const char *const critical_name[]=    { "Payload-Critical", NULL };
+const char *const no_critical_name[]= { "Payload-Non-Critical", NULL };
+enum_names no_crit_names ={ ISAKMP_PAYLOAD_NONCRITICAL,
+			    ISAKMP_PAYLOAD_NONCRITICAL,
+			    no_critical_name, NULL};
+enum_names critical_names = 
+{ ISAKMP_PAYLOAD_CRITICAL, ISAKMP_PAYLOAD_CRITICAL,
+  critical_name, &no_crit_names};
+
+/* Transform-type Encryption */
+const char *const trans_type_encr_name[]={
+    "des-iv64",
+    "des",
+    "3des",
+    "rc5",
+    "idea",
+    "cast",
+    "blowfish",
+    "3idea",
+    "des-iv32",
+    "res10",
+    "null",
+    "aes-cbc",
+    "aes-ctr",
+};
+enum_names trans_type_encr_names =
+{ IKEv2_ENCR_DES_IV64, IKEv2_ENCR_AES_CTR, trans_type_encr_name, NULL};
+
+/* Transform-type PRF */
+const char *const trans_type_prf_name[]={
+    "prf-hmac-md5",
+    "prf-hmac-sha1",
+    "prf-hmac-tiger",
+    "prf-hmac-aes128-xcbc",
+};
+enum_names trans_type_prf_names =
+{ IKEv2_PRF_HMAC_MD5, IKEv2_PRF_AES128_XCBC, trans_type_prf_name, NULL};
+
+/* Transform-type Integrity */
+const char *const trans_type_integ_name[]={
+    "auth-hmac-md5-96",
+    "auth-hmac-sha1-96",
+    "auth-des-mac",
+    "auth-kpdk-md5",
+    "auth-aes-xcbc-96",
+};
+enum_names trans_type_integ_names =
+{ IKEv2_AUTH_HMAC_MD5_96, IKEv2_AUTH_AES_XCBC_96, trans_type_integ_name, NULL};
+
+/* Transform-type Integrity */
+const char *const trans_type_esn_name[]={
+    "esn-disabled",
+    "esn-enabled",
+};
+enum_names trans_type_esn_names =
+{ IKEv2_ESN_DISABLED, IKEv2_ESN_ENABLED, trans_type_esn_name, NULL};
+
+/* Transform Type */
+const char *const trans_type_name[]={
+    "trans-type-encr",
+    "trans-type-prf",
+    "trans-type-integ",
+    "trans-type-dh",
+    "trans-type-esn"
+};
+enum_names trans_type_names =
+{ IKEv2_TRANS_TYPE_ENCR, IKEv2_TRANS_TYPE_ESN, trans_type_name, NULL};
+
+/* for each IKEv2 transform attribute,which enum_names describes its values? */
+enum_names *ikev2_transid_val_descs[] = {
+    NULL,
+    &trans_type_encr_names, /* 1 */
+    &trans_type_prf_names,  /* 2 */
+    &trans_type_integ_names, /* 3 */
+    &oakley_group_names,    /* 4 */
+    &trans_type_esn_names,  /* 5 */
+};
+const unsigned int ikev2_transid_val_descs_size = elemsof(ikev2_transid_val_descs);
+    
+
 /* socket address family info */
 
 static const char *const af_inet_name[] = {
