@@ -1,4 +1,4 @@
- /* 
+/* 
  * IKEv2 parent SA creation routines
  * Copyright (C) 2007  Michael Richardson <mcr@xelerance.com>
  *
@@ -32,7 +32,8 @@
 #include "connections.h"	
 
 #include "crypto.h" /* requires sha1.h and md5.h */
-
+#include "x509.h"
+#include "x509more.h"
 #include "ike_alg.h"
 #include "kernel_alg.h"
 #include "plutoalg.h"
@@ -167,7 +168,7 @@ ikev2parent_outI1(int whack_sock
 	}
     }
     if(groupnum == 0) {
-	groupnum = OAKLEY_GROUP_MODP1536;
+	groupnum = OAKLEY_GROUP_MODP2048;
     }
     st->st_oakley.group=lookup_group(groupnum); 
 
@@ -680,7 +681,7 @@ stf_status ikev2parent_inR1outI2(struct msg_digest *md)
 
     DBG(DBG_CONTROLMORE
 	, DBG_log("ikev2 parent inR1: calculating g^{xy} in order to send I2"));
-  
+ 
     /* KE in */
     keyex_pbs = &md->chain[ISAKMP_NEXT_v2KE]->pbs;
     RETURN_STF_FAILURE(accept_KE(&st->st_gr, "Gr", st->st_oakley.group, keyex_pbs));
@@ -1026,6 +1027,7 @@ ikev2_parent_inR1outI2_tail(struct pluto_crypto_req_cont *pcrc
     unsigned char *idhash;
     unsigned char *authstart;
     struct state *pst = st;
+    bool send_cert = FALSE;
 
     finish_dh_v2(st, r);
 
@@ -1098,8 +1100,15 @@ ikev2_parent_inR1outI2_tail(struct pluto_crypto_req_cont *pcrc
 	hmac_init_chunk(&id_ctx, pst->st_oakley.integ_hasher, pst->st_skey_pi);
 	build_id_payload((struct isakmp_ipsec_id *)&r_id, &id_b, &c->spd.this);
 	r_id.isai_critical = ISAKMP_PAYLOAD_CRITICAL;
-	r_id.isai_np = ISAKMP_NEXT_v2AUTH;
-
+	{  /* decide to send CERT payload */
+	    send_cert = doi_send_ikev2_cert_thinking(st);
+	    
+	    if(send_cert) 
+		r_id.isai_np = ISAKMP_NEXT_v2CERT;
+	    else  
+		r_id.isai_np = ISAKMP_NEXT_v2AUTH; 
+	}
+	
 	id_start = e_pbs_cipher.cur;
 	if (!out_struct(&r_id
 			, &ikev2_id_desc
@@ -1120,7 +1129,19 @@ ikev2_parent_inR1outI2_tail(struct pluto_crypto_req_cont *pcrc
 	hmac_update(&id_ctx, id_start, id_len);
 	idhash = alloca(pst->st_oakley.integ_hasher->hash_digest_len);
 	hmac_final(idhash, &id_ctx);
-    }
+    } 
+
+    /* send [CERT,] payload RFC 4306 3.6, 1.2) */
+    {
+	
+	if(send_cert) {
+	    stf_status certstat = ikev2_send_cert( st, md
+	    					   , INITIATOR
+						   ,ISAKMP_NEXT_v2AUTH
+						   , &e_pbs_cipher);
+	    if(certstat != STF_OK) return certstat;
+	}
+    } 
 
     /* send out the AUTH payload */
     {
@@ -1217,7 +1238,7 @@ stf_status ikev2parent_inI2outR2(struct msg_digest *md)
 
     DBG(DBG_CONTROLMORE
 	, DBG_log("ikev2 parent inI2outR2: calculating g^{xy} in order to decrypt I2"));
-
+    
     /* verify that there is in fact an encrypted payload */
     if(!md->chain[ISAKMP_NEXT_v2E]) {
 	openswan_log("R2 state should receive an encrypted payload");
@@ -1328,6 +1349,28 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	hmac_final(idhash_in, &id_ctx);
     }
 
+    /* process CERT payload */
+    {
+    if(md->chain[ISAKMP_NEXT_v2CERT])
+	{
+	    /* should we check if we should accept a cert payload ?
+	     *  has_preloaded_public_key(st)
+	     */ 
+	    /* in v1 code it is  decode_cert(struct msg_digest *md) */
+	    DBG(DBG_CONTROLMORE
+		, DBG_log("has a v2_CERT payload going to process it "));	  
+	    ikev2_decode_cert(md); 
+	}
+     }
+
+    /* process CERTREQ payload */
+    if(md->chain[ISAKMP_NEXT_v2CERTREQ]) 
+    {
+	    DBG(DBG_CONTROLMORE
+		,DBG_log("has a v2CERTREQ payload going to decode it"));
+	    ikev2_decode_cr(md, &st->st_connection->requested_ca);
+    }
+
     /* process AUTH payload */
     if(!md->chain[ISAKMP_NEXT_v2AUTH]) {
 	openswan_log("no authentication payload found");
@@ -1424,6 +1467,7 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	e_pbs_cipher.cur = e_pbs.cur;
 	encstart = e_pbs_cipher.cur;
 	
+         bool send_cert = FALSE;
 	/* send out the IDr payload */
 	{
 	    struct ikev2_id r_id;
@@ -1438,7 +1482,14 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	    build_id_payload((struct isakmp_ipsec_id *)&r_id, &id_b,
 			     &c->spd.this);
 	    r_id.isai_critical = ISAKMP_PAYLOAD_CRITICAL;
-	    r_id.isai_np = ISAKMP_NEXT_v2AUTH;
+           {  /* decide to send CERT payload */
+	    send_cert = doi_send_ikev2_cert_thinking(st);
+	    
+	    if(send_cert) 
+		r_id.isai_np = ISAKMP_NEXT_v2CERT;
+	    else  
+		r_id.isai_np = ISAKMP_NEXT_v2AUTH; 
+	    }
 	    id_start = e_pbs_cipher.cur;
 	    
 	    if (!out_struct(&r_id
@@ -1460,6 +1511,20 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	    hmac_final(idhash_out, &id_ctx);
 	}
 
+	DBG(DBG_CONTROLMORE
+	    , DBG_log("assembeled IDr payload"));
+	
+	/* send CERT payload RFC 4306 3.6, 1.2:([CERT,] ) */
+	{
+	    if(send_cert){
+		stf_status certstat = ikev2_send_cert( st, md
+						       , RESPONDER
+						       ,ISAKMP_NEXT_v2AUTH
+						       , &e_pbs_cipher);
+	    	if(certstat != STF_OK) return certstat;
+	    }
+    	} 
+
 	/* authentication good, see if there is a child SA available */
 	if(md->chain[ISAKMP_NEXT_v2SA] == NULL
 	   || md->chain[ISAKMP_NEXT_v2TSi] == NULL
@@ -1471,7 +1536,9 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	} else {
 	    np = ISAKMP_NEXT_v2SA;
 	}
-
+	
+	DBG(DBG_CONTROLMORE
+	    , DBG_log("going to assmemble AUTH payload"));	  
 	/* now send AUTH payload */
 	{
 	    stf_status authstat = ikev2_send_auth(c, st
@@ -1582,6 +1649,18 @@ stf_status ikev2parent_inR2(struct msg_digest *md)
 	idhash_in = alloca(pst->st_oakley.integ_hasher->hash_digest_len);
 	hmac_final(idhash_in, &id_ctx);
     }
+    {
+    if(md->chain[ISAKMP_NEXT_v2CERT])
+	{
+	    /* should we check if we should accept a cert payload ?
+	     *  has_preloaded_public_key(st)
+	     */ 
+	    /* in v1 code it is  decode_cert(struct msg_digest *md) */
+	    DBG(DBG_CONTROLMORE
+		, DBG_log("has a v2_CERT payload going to decode it"));	  
+	    ikev2_decode_cert(md); 
+	}
+     }
 
     /* process AUTH payload */
     if(!md->chain[ISAKMP_NEXT_v2AUTH]) {
