@@ -8,9 +8,8 @@ iptables -A INPUT -i eth1 -s 192.0.2.0/24 -j DROP
 ping -n -c 4 192.0.2.254
 
 ipsec setup start
+/testing/pluto/bin/wait-until-pluto-started
 ipsec auto --add westnet-eastnet-x509
-/testing/pluto/basic-pluto-01/eroutewait.sh trap
-ipsec auto --up  westnet-eastnet-x509
 
 echo done
 
